#ifndef __HTTP_DOWNLOAD_PRIVATE_H__
#define __HTTP_DOWNLOAD_PRIVATE_H__

#if defined(__cplusplus)
extern "C" {
#endif

#include "aiot_ota_api.h"

typedef enum {
    DOWNLOAD_STATUS_START,
    DOWNLOAD_STATUS_FETCH,
    DOWNLOAD_STATUS_RENEWAL,
} download_status_t;

/**
 * @brief 处理下载任务的句柄, 该句柄主要用于通过http协议从指定的url下载固件
 *
 */
typedef struct {
    void
    *userdata;       /* 组件调用recv_handler 时的入参之一, 传入用户数据 */
    aiot_download_recv_handler_t       recv_handler;    /* 设备收到分段的固件报文时的回调函数 */
    aiot_download_task_desc_t          *task_desc;      /* 某次下载活动的目标描述信息, 如URL等 */
    aiot_sysdep_portfile_t             *sysdep;
    uint32_t                           range_start;
    uint32_t                           range_end;

    /*---- 以上都是用户在API可配 ----*/
    /*---- 以下都是downloader内部使用, 用户无感知 ----*/

    uint8_t         download_status;
    void            *http_handle;
    uint32_t        size_fetched;
    uint32_t        range_size_fetched;
    uint32_t        content_len;
    int32_t         percent;
    int32_t         http_rsp_status_code;
    void            *digest_ctx;
    void            *data_mutex;
    void            *recv_mutex;
} download_handle_t;

#if defined(__cplusplus)
}
#endif

#endif  /* __OTA_PRIVATE_H__ */