#ifndef _MQTT_IMPL_H_
#define _MQTT_IMPL_H_

#if defined(__cplusplus)
extern "C" {
#endif

/**
 *
 * MQTT3.1 Fixed Header
 * | Bit       |          7 ~ 4           |            3 ~ 0            |
 * | byte 1    | MQTT Control Packet Type | Flags specific to each Type |
 * | byte 2... |             Remaining Length(1 ~ 4 bytes)              |
 *
 * MQTT3.1 UTF-8 Encoded Strings
 * | Bit       |                         7 ~ 0                          |
 * | byte 1    |                   String Length MSB                    |
 * | byte 2    |                   String Length LSB                    |
 * | byte 3... |              UTF-8 Encoded Character Data              |
 *
 */
#define CORE_QOS0                                   (0x00)
#define CORE_QOS1                                   (0x01)
#define CORE_MQTT_FIXED_HEADER_LEN                  (1)
#define CORE_MQTT_REMAINLEN_MAXLEN                  (4)
#define CORE_MQTT_UTF8_STR_EXTRA_LEN                (2)
#define CORE_MQTT_PACKETID_LEN                      (2)
#define CORE_MQTT_REQUEST_QOS_LEN                   (1)

/* MQTT 3.1 Connect Packet */
#define CORE_MQTT_CONN_PKT_TYPE                     (0x10)
#define CORE_MQTT_CONN_FIXED_HEADER_LEN             (CORE_MQTT_FIXED_HEADER_LEN) /* value: 0x10 */
#define CORE_MQTT_CONN_REMAINLEN_MAXLEN             (CORE_MQTT_REMAINLEN_MAXLEN)
#define CORE_MQTT_CONN_PROTOCOL_NAME_LEN            (6) /* value: 0x00, 0x04, 0x4D, 0x51, 0x54, 0x54 */
#define CORE_MQTT_CONN_PROTOCOL_LEVEL_LEN           (1) /* value: 0x04 */
#define CORE_MQTT_CONN_CONNECT_FLAG_LEN             (1) /* value: 0xC0 */
#define CORE_MQTT_CONN_KEEP_ALIVE_LEN               (2) /* value: CORE_MQTT_conn_pkt_t.keep_alive */
#define CORE_MQTT_CONN_FIXED_HEADER_TOTAL_LEN       (CORE_MQTT_CONN_FIXED_HEADER_LEN + \
        CORE_MQTT_CONN_REMAINLEN_MAXLEN + \
        CORE_MQTT_CONN_PROTOCOL_NAME_LEN + \
        CORE_MQTT_CONN_PROTOCOL_LEVEL_LEN + \
        CORE_MQTT_CONN_CONNECT_FLAG_LEN + \
        CORE_MQTT_CONN_KEEP_ALIVE_LEN)
#define CORE_MQTT_CONN_REMAINLEN_FIXED_LEN          (CORE_MQTT_CONN_PROTOCOL_NAME_LEN + \
        CORE_MQTT_CONN_PROTOCOL_LEVEL_LEN + \
        CORE_MQTT_CONN_CONNECT_FLAG_LEN + \
        CORE_MQTT_CONN_KEEP_ALIVE_LEN)

/* MQTT 3.1 Connect ACK Packet */
#define CORE_MQTT_CONNACK_PKT_TYPE                  (0x20)
#define CORE_MQTT_CONNACK_FIXED_HEADER_LEN          (CORE_MQTT_FIXED_HEADER_LEN) /* value: 0x20 */
#define CORE_MQTT_CONNACK_REMAINLEN_MAXLEN          (1) /* value: 0x02 */
#define CORE_MQTT_CONNACK_FLAGS_LEN                 (1) /* value: 0x00 */
#define CORE_MQTT_CONNACK_RETURN_CODE_LEN           (1)
#define CORE_MQTT_CONNACK_FIXED_HEADER_TOTAL_LEN    (CORE_MQTT_CONNACK_FIXED_HEADER_LEN + \
        CORE_MQTT_CONNACK_REMAINLEN_MAXLEN + \
        CORE_MQTT_CONNACK_FLAGS_LEN + \
        CORE_MQTT_CONNACK_RETURN_CODE_LEN)

#define CORE_MQTT_CONNACK_RCODE_ACCEPTED                            (0x00)
#define CORE_MQTT_CONNACK_RCODE_UNACCEPTABLE_PROTOCOL_VERSION       (0x01)
#define CORE_MQTT_CONNACK_RCODE_SERVER_UNAVAILABLE                  (0x02)
#define CORE_MQTT_CONNACK_RCODE_BAD_USERNAME_PASSWORD               (0x03)
#define CORE_MQTT_CONNACK_RCODE_NOT_AUTHORIZED                      (0x04)

#define CORE_MQTT_DYNREG_CONNACK_RCODE_ACCEPTED                     (0x00)
#define CORE_MQTT_DYNREG_CONNACK_RCODE_IDENTIFIER_REJECTED          (0x02)
#define CORE_MQTT_DYNREG_CONNACK_RCODE_SERVER_UNAVAILABLE           (0x03)
#define CORE_MQTT_DYNREG_CONNACK_RCODE_BAD_USERNAME_PASSWORD        (0x04)

/* MQTT 3.1 Disconnect Packet */
#define CORE_MQTT_DISCONNECT_PKT_TYPE               (0xE0)

/* MQTT 3.1 Ping Request Packet */
#define CORE_MQTT_PINGREQ_PKT_TYPE                  (0xC0)

/* MQTT 3.1 Ping Response Packet */
#define CORE_MQTT_PINGRESP_PKT_TYPE                 (0xD0)
#define CORE_MQTT_PINGRESP_FIXED_HEADER_LEN         (CORE_MQTT_FIXED_HEADER_LEN) /* value: 0xD0 */
#define CORE_MQTT_PINGRESP_REMAINLEN_MAXLEN         (1) /* value: 0x00 */

/* MQTT 3.1 Publish Packet */
#define CORE_MQTT_PUBLISH_PKT_TYPE                  (0x30)
#define CORE_MQTT_PUBLISH_TOPICLEN_LEN              (2)

/* MQTT 3.1 Publish ACK Packet */
#define CORE_MQTT_PUBACK_PKT_TYPE                   (0x40)

/* MQTT 3.1 Subscribe Packet */
#define CORE_MQTT_SUB_PKT_TYPE                      (0x80)
#define CORE_MQTT_SUB_PKT_RESERVE                   (0x02)
#define CORE_MQTT_UNSUB_PKT_RESERVE                 (0x02)

/* MQTT 3.1 Subscribe ACK Packet */
#define CORE_MQTT_SUBACK_PKT_TYPE                   (0x90)

#define CORE_MQTT_SUBACK_RCODE_MAXQOS0              (0x00)
#define CORE_MQTT_SUBACK_RCODE_MAXQOS1              (0x01)
#define CORE_MQTT_SUBACK_RCODE_MAXQOS2              (0x02)
#define CORE_MQTT_SUBACK_RCODE_FAILURE              (0x80)

/* MQTT 3.1 Unsubscribe Packet */
#define CORE_MQTT_UNSUB_PKT_TYPE                    (0xA0)

/* MQTT 3.1 Unsubscribe ACK Packet */
#define CORE_MQTT_UNSUBACK_PKT_TYPE                 (0xB0)

/* MQTT 3.1 unimplemented Packet */
#define CORE_MQTT_PUBREC_PKT_TYPE                   (0x50)
#define CORE_MQTT_PUBREL_PKT_TYPE                   (0x60)
#define CORE_MQTT_PUBCOMP_PKT_TYPE                  (0x70)

typedef struct {
    void *network_handle;
    uint32_t send_timeout_ms;
    uint32_t recv_timeout_ms;
    aiot_mqtt_recv_handler_t recv_handler;
    aiot_mqtt_event_handler_t event_handler;
    void *userdata;
    uint16_t packet_id;
    void *data_mutex;
} mqtt_client_t;


#if defined(__cplusplus)
}
#endif

#endif
