
/**
 * @file aiot_MQTT_PORT_api.h
 * @brief SDK Core系统依赖头文件, 所有Core中的系统依赖均在此文件中列出
 * @date 2019-12-27
 *
 * @copyright Copyright (C) 2015-2018 Alibaba Group Holding Limited
 *
 */

#ifndef _AIOT_MQTT_PORT_API_H_
#define _AIOT_MQTT_PORT_API_H_

#if defined(__cplusplus)
extern "C" {
#endif

#include <stdint.h>
#include "aiot_sysdep_api.h"
#include "aiot_mqtt_api.h"

typedef struct {
    char *username;
    char *password;
    char *clientid;
    char *host;
    uint16_t port;
    aiot_sysdep_network_cred_t *cred;
    char *psk_id;
    char *psk;
    uint16_t keep_alive_s;
    uint8_t  clean_session;
    uint32_t connect_timeout_ms;
    uint32_t send_timeout_ms;
    uint32_t recv_timeout_ms;
    aiot_mqtt_event_handler_t event_handler;
    void *userdata;
} conn_para_t;

typedef struct {
    uint16_t reserve;
} disconn_para_t;

typedef struct {
    char *topic;
    uint8_t qos;
} sub_para_t;

typedef struct {
    char *topic;
} unsub_para_t;

typedef struct {
    char *topic;
    uint8_t *payload;
    uint32_t payload_len;
    uint8_t qos;
    uint16_t packet_id; /* qos0时该值为0，qos1消息重发时该值不为0 */
} pub_para_t;

/**
 * @brief mqtt实现层抽象接口
 */
typedef struct {
    /* 
    输出：mqtt实现句柄，保存上下文
    */
    void* (*mqtt_init)();
    /* 
    输入：
        建连信息：host、port 
            mqtt必选配置：username、password、client_id、cleansession、keepalive
            mqtt可选配置：版本、v5特性、遗嘱、超时时间
        回调函数：状态回调、消息回调
    输出：mqtt实现句柄，保存上下文
    */
    int32_t (*mqtt_connect)(void *handle, conn_para_t *params);

    /* 
    输入：
        mqtt句柄
        v5特性
    输出：成功/错误
    */
    int32_t (*mqtt_disconnect)(void *handle, disconn_para_t *params);

    /* 
    输入：
        mqtt句柄
        topic
        qos
        v5特性
    输出：成功/错误
    */
    int32_t (*mqtt_sub)(void *handle, sub_para_t *params);
    /* 
    输入：
        mqtt句柄
        topic
        v5特性
    输出：成功/错误
    */
    int32_t (*mqtt_unsub)(void *handle, unsub_para_t *params);

    /* 
    输入：
        mqtt句柄
        topic
        payload
        retain
        qos
        v5特性
    输出：成功/错误
    */
    int32_t (*mqtt_pub)(void *handle, pub_para_t *params);

    /* 
    输入：
        mqtt句柄
    输出：成功/错误
    */
    int32_t (*mqtt_heartbeat)(void *handle);

    /* 
    输入：
        mqtt句柄，消息id
    输出：成功/错误
    */
    int32_t (*mqtt_puback)(void *handle, uint16_t packet_id);

    /* 
    输入：
        mqtt句柄
    输出：成功/错误
    */
    int32_t (*mqtt_recv)(void *handle, aiot_mqtt_recv_t *packet);

    /* 
    输入：
        mqtt句柄
    输出：成功/错误
    */
    int32_t (*mqtt_deinit)(void **handle);
} aiot_mqtt_port_t;

#if defined(__cplusplus)
}
#endif

#endif
